@@ -6,6 +6,8 @@ import functools
 from datetime import date
 from timeit import default_timer
 
+from ..conditional import is_tty_console
+from ..conditional import is_utf_console
 from ..conditional import ultrajson_installed
 
 if ultrajson_installed:
@@ -13,6 +15,11 @@ if ultrajson_installed:
 else:
     import json
 
+if is_utf_console:
+    __vengeance_prefix__ = (' '*4) + 'ν: '    # nu: chr(957)
+else:
+    __vengeance_prefix__ = (' '*4) + 'v: '    # ascii
+
 
 def print_runtime(f=None):
 
@@ -24,8 +31,8 @@ def print_runtime(f=None):
             toc  = default_timer()
             elapsed = -(tic - toc)
 
-            vengeance_message('@{}: {}'.format(function_name(_f_),
-                                               format_seconds(elapsed)))
+            print(vengeance_message('@{}: {}'.format(function_name(_f_),
+                                                     format_seconds(elapsed))))
             return retv
 
         return functools_wrapper
@@ -68,14 +75,17 @@ def print_performance(f=None, *, repeat=3):
                 else:
                     worst = max(worst, elapsed)
 
-            vengeance_message('@{}'
-                              '\n\t\tworst:   {}'
-                              '\n\t\taverage: {}'
-                              '\n\t\tbest:    {}'
-                              .format(function_name(_f_),
-                                      format_milliseconds(worst),
-                                      format_milliseconds(total / repeat),
-                                      format_milliseconds(best)))
+            s = ('@{}'
+                 '\n        best:    {}'
+                 '\n        average: {}'
+                 '\n        worst:   {}'
+                 .format(function_name(_f_),
+                         format_milliseconds(best),
+                         format_milliseconds(total / repeat),
+                         format_milliseconds(worst)))
+            s = vengeance_message(s)
+
+            print(s)
 
             if was_gc_enabled:
                 gc.enable()
@@ -90,17 +100,78 @@ def print_performance(f=None, *, repeat=3):
         return performance_wrapper(f)
 
 
+# noinspection DuplicatedCode
+def styled(message,
+           color_style=None,
+           effect_style=None):
+
+    # region {escape codes}
+    effect_codes = {'bold':      '\x1b[1m',
+                    'italic':    '\x1b[3m',
+                    'underline': '\x1b[4m',
+                    'end':       '\x1b[0m',
+                    '':          '',
+                    None:        ''}
+    color_codes = {'grey':           '\x1b[29m',
+                   'white':          '\x1b[30m',
+                   'red':            '\x1b[31m',
+                   'orange':         '\x1b[32m',
+                   'yellow':         '\x1b[33m',
+                   'blue':           '\x1b[34m',
+                   'magenta':        '\x1b[35m',
+                   'green':          '\x1b[36m',
+                   'bronze':         '\x1b[37m',
+                   'bright red':     '\x1b[91m',
+                   'bright yellow':  '\x1b[93m',
+                   'bright magenta': '\x1b[95m',
+                   'bright cyan':    '\x1b[96m',
+                   '':               '',
+                   None:             ''}
+    # endregion
+
+    if color_style not in color_codes:
+        raise KeyError('invalid color: {}'.format(color_style))
+    if effect_style not in effect_codes:
+        raise KeyError('invalid style: {}'.format(effect_style))
+
+    if is_tty_console:          # TTY console doesn't support ascii escapes
+        return message
+
+    effect_end     = effect_codes['end']
+    styled_message = ('{ascii_color}{effect_start}{message}{effect_end}'
+                       .format(ascii_color=color_codes[color_style],
+                               effect_start=effect_codes[effect_style],
+                               message=message,
+                               effect_end=effect_end))
+    return styled_message
+
+
+def vengeance_message(message):
+    return __vengeance_prefix__ + message
+
+
+def print_u(message):
+    try:
+        print(message)
+    except UnicodeEncodeError:
+        print(unicode_to_ascii(message))
+
+
+def unicode_to_ascii(message):
+    return (message.encode('ascii', errors='backslashreplace')
+                   .decode('ascii'))
+
+
 def deprecated(message='deprecated'):
 
     def deprecated_wrapper(_f_):
         @functools.wraps(_f_)
         def functools_wrapper(*args, **kwargs):
-            _message_ = '@{}():  "{}"'.format(function_name(_f_), message)
-
+            _message_ = "@{}: '{}'".format(function_name(_f_), message)
             vengeance_warning(_message_,
                               DeprecationWarning,
                               stacklevel=3,
-                              stack_frame=None)
+                              stackframe=None)
 
             return _f_(*args, **kwargs)
 
@@ -112,65 +183,50 @@ def deprecated(message='deprecated'):
 def vengeance_warning(message,
                       category=Warning,
                       stacklevel=2,
-                      stack_frame=None):
-    import inspect
-    import sys
-    import traceback
+                      stackframe=None):
+    # import sys
     import warnings
 
-    original_formatwarning = warnings.formatwarning
-
     # region {closure functions}
-    if stack_frame is None:
-        stack_frame = traceback.extract_stack(inspect.currentframe(), limit=stacklevel)[0]
+    if isinstance(stacklevel, int) and stackframe is None:
+        import inspect
+        import traceback
+        stackframe = traceback.extract_stack(inspect.currentframe(), limit=stacklevel)[0]
 
-    def vengeance_formatwarning(*_):
+    def vengeance_formatwarning(*_, **__):
+        nonlocal stackframe
         nonlocal category
         nonlocal message
-        nonlocal stack_frame
+
+        aligned_indent = ' ' * len(__vengeance_prefix__)
 
         w_category = object_name(category)
-        w_message  = message.replace('"', '').replace('\n', '\n\t   ')
+        w_message  = message.replace('\n', '\n' + aligned_indent)
 
-        w_s = ('\tν: <{w_category}> {w_message}'
-               '\n\tν: File "{filename}", line {lineno}'
-               '\n\n'.format(w_category=w_category,
-                             w_message=w_message,
-                             filename=stack_frame.filename,
-                             lineno=stack_frame.lineno))
-        return w_s
+        line_1 = '<{}> {}'.format(w_category, w_message)
+        if stackframe is None:
+            line_2 = 'File {unknown}, line {unknown}'
+        else:
+            line_2 = 'File "{}", line {}'.format(stackframe.filename, stackframe.lineno)
+
+        line_1 = vengeance_message(line_1)
+        line_2 = aligned_indent + line_2
+        w_message = '{}\n{}\n'.format(line_1, line_2)
+
+        # w_message = styled(w_message, 'yellow', 'bold')
+
+        return w_message
     # endregion
 
+    original_formatwarning = warnings.formatwarning
+
     warnings.formatwarning = vengeance_formatwarning
     warnings.warn(message)
     warnings.formatwarning = original_formatwarning
 
-    sys.stdout.flush()
-    sys.stderr.flush()
-
-
-def vengeance_message(s, printed=True):
-    vs = '\tν: {}'.format(s)
-    if printed:
-        print_unicode(vs)
-
-    return vs
-
-
-def print_unicode(s):
-    # region {closure functions}
-    def convert_unicode(_s_):
-        _s_ = (_s_.replace('ν', 'v')
-                  .replace('μ', 'u'))
-        _s_ = (_s_.encode('ascii', errors='backslashreplace')
-                  .decode('ascii'))
-        return _s_
-    # endregion
-
-    try:
-        print(s)
-    except UnicodeEncodeError:
-        print(convert_unicode(s))
+    # print(end='')
+    # sys.stdout.flush()
+    # sys.stderr.flush()
 
 
 def format_seconds(secs):
@@ -197,7 +253,10 @@ def format_milliseconds(ms):
     elif ms >= 1:
         f_ms = '{:.1f} ms'.format(ms)
     elif us >= 1:
-        f_ms = '{:.0f} μs'.format(us)
+        if is_utf_console:
+            f_ms = '{:.0f} μs'.format(us)
+        else:
+            f_ms = '{:.0f} mi s'.format(us)
     else:
         f_ms = '{:.0f} ns'.format(ns)
 
